"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
/**
 * Base class for all code writers.
 *
 * Defines a common constructor signature, and slots/hooks that
 * will be called at the appropriate times during processing.
 */
class Writer {
    /**
     * Create a code writer.
     *
     * @param output {string} The output path to write to. Either an existing directory, or a file path
     */
    constructor(output, options) {
        /**
         * Helper for single output processing, indicating whether the first
         * widget is being processed. Sub-classes should set this to false
         * after the first time onWidget is called.
         */
        this.firstWidget = true;
        this.output = output;
        this.outputMultiple = fs.existsSync(output) && fs.statSync(output).isDirectory();
    }
    /**
     * Slot/hook to be called when all widgets have been processed.
     *
     * The default implementation is a no-op.
     *
     * @memberof Writer
     */
    finalize() {
    }
    ;
}
exports.Writer = Writer;
//# sourceMappingURL=base.js.map